import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.assertj.core.util.Files;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import ru.ulstu.core.error.XlsParseException;
import ru.ulstu.core.service.XlsDocumentBuilder;

import java.io.*;

public class XlsDocumentBuilderTest {
    private final static String TEST_FILE_NAME = "testDoc.xlsx";
    private final static String TEST_XLS_FILE_NAME = "testDoc.xls";
    private final static String CELL_TEST_VALUE = "cell test value";

    @Before
    public void before() {
        Files.delete(new File(TEST_FILE_NAME));
        Files.delete(new File(TEST_XLS_FILE_NAME));
    }

    @Test
    public void documentSaveTest() throws IOException, XlsParseException {
        File xlsFile = new File(TEST_FILE_NAME);
        XlsDocumentBuilder xlsDocumentBuilder = new XlsDocumentBuilder(xlsFile);
        xlsDocumentBuilder.save();
        File savedFile = new File(TEST_FILE_NAME);
        XSSFWorkbook workbook = new XSSFWorkbook(new FileInputStream(savedFile));
        Assert.assertEquals(workbook.getActiveSheetIndex(), 0);
    }

    @Test
    public void insertSheetTest() throws IOException, XlsParseException {
        File xlsFile = new File(TEST_FILE_NAME);
        XlsDocumentBuilder xlsDocumentBuilder = new XlsDocumentBuilder(xlsFile)
                .insertNewSheet("new sheet");
        Assert.assertEquals(xlsDocumentBuilder.getSheetCount(), 2);
        Assert.assertEquals(xlsDocumentBuilder.getActiveSheetIndex(), 1);
    }

    @Test
    public void openExistingFile() throws IOException, XlsParseException {
        File xlsFile = new File(TEST_FILE_NAME);
        new XlsDocumentBuilder(xlsFile)
                .insertNewSheet("new sheet")
                .save();

        XlsDocumentBuilder xlsDocumentBuilder = new XlsDocumentBuilder(xlsFile);
        Assert.assertEquals(xlsDocumentBuilder.getSheetCount(), 2);
    }

    @Test
    public void openExistingXlsFile() throws IOException, XlsParseException {
        File xlsFile = new File(TEST_XLS_FILE_NAME);
        Workbook workbook = new HSSFWorkbook();
        workbook.createSheet("123");
        OutputStream out = new FileOutputStream(xlsFile);
        workbook.write(out);

        XlsDocumentBuilder xlsDocumentBuilder = new XlsDocumentBuilder(xlsFile);
        Assert.assertEquals(xlsDocumentBuilder.getSheetCount(), 1);
    }

    @Test
    public void openWrongXlsFile() throws IOException, XlsParseException {
        File xlsFile = new File(TEST_XLS_FILE_NAME);
        OutputStream out = new FileOutputStream(xlsFile);
        out.write(1);
        out.close();

        boolean wrongDocument = false;
        try {
            new XlsDocumentBuilder(xlsFile);
        } catch (XlsParseException ex) {
            wrongDocument = true;
        }
        Assert.assertTrue(wrongDocument);
    }

    @Test
    public void setActiveSheetTest() throws IOException, XlsParseException {
        File xlsFile = new File(TEST_FILE_NAME);
        XlsDocumentBuilder xlsDocumentBuilder = new XlsDocumentBuilder(xlsFile)
                .insertNewSheet("new sheet")
                .setActiveSheet(0);
        Assert.assertEquals(xlsDocumentBuilder.getActiveSheetIndex(), 0);
    }

    @Test
    public void getRowCountTest() throws IOException, XlsParseException {
        File xlsFile = new File(TEST_FILE_NAME);
        XlsDocumentBuilder xlsDocumentBuilder = new XlsDocumentBuilder(xlsFile)
                .insertNewSheet("new sheet");
        Assert.assertEquals(xlsDocumentBuilder.getRowCount(), 0);
        xlsDocumentBuilder.setCellValue(10, 10, CELL_TEST_VALUE);
        Assert.assertEquals(xlsDocumentBuilder.getRowCount(), 10);
    }

    @Test
    public void getColumnCountTest() throws IOException, XlsParseException {
        File xlsFile = new File(TEST_FILE_NAME);
        XlsDocumentBuilder xlsDocumentBuilder = new XlsDocumentBuilder(xlsFile)
                .insertNewSheet("new sheet");
        Assert.assertEquals(xlsDocumentBuilder.getColumnCount(), 0);
        xlsDocumentBuilder.setCellValue(10, 10, CELL_TEST_VALUE);
        Assert.assertEquals(xlsDocumentBuilder.getColumnCount(), 10);
    }

    @Test
    public void getCellAsStringTest() throws IOException, XlsParseException {
        File xlsFile = new File(TEST_FILE_NAME);
        XlsDocumentBuilder xlsDocumentBuilder = new XlsDocumentBuilder(xlsFile)
                .insertNewSheet("new sheet");
        xlsDocumentBuilder.setCellValue(10, 10, CELL_TEST_VALUE);
        Assert.assertEquals(xlsDocumentBuilder.getCellAsString(10, 10), CELL_TEST_VALUE);
    }

    @After
    public void after() {
        Files.delete(new File(TEST_FILE_NAME));
        Files.delete(new File(TEST_XLS_FILE_NAME));
    }
}
